/*******************************************************************************
*  TestFilter.h - Exorcise Regexpes against a log
*
*  Berzerk/
*
*  Part of the BuckoSoft Network Monitor
*  Copyright(c) 2016 - BuckoSoft, Inc.
*
*  http://git.buckosoft.com/gitweb/?p=Berzerk.git;a=history;f=Berzerk/TestFilter.cpp
*
\*******************************************************************************/

#include "stdafx.h"
#include "../config.h"
#include <iostream>
#include <fstream>
#include <boost/foreach.hpp>

#include "server/urlencode.h"
#include "WebHandler.h"
#include "TestFilter.h"

using namespace std;

namespace berzerk {

PCSTR	s_matched	= "Matched";
PCSTR	s_line		= "Line";

TestFilter::TestFilter(const LogFile* _logFile, const string& _regexp, const string& _regexpIP) {
	logFile = _logFile;
	regexp = _regexp;
	regexpIP = _regexpIP;
	maxLines = 50;
	//hasError = false;
}

///////////////////////////////////////////////////////////////////////////////
bool	TestFilter::run() {
	char		buff[4096];

//	matchedLines.push_back(TLine("Hi momBad text c++ha how are you? 1.2.3.4", 6, 8, 34, 7));
//	matchedLines.push_back(TLine("Hi momBad text c++ha how are you? 1.2.3.4 abcd", 6, 8, 34, 7));
//	matchedLines.push_back(TLine("Hi dad 5.6.7.8 Bad text", 15, 8, 7, 7));
//	return(true);
	ifstream file;
	file.open(logFile->name.c_str() , ifstream::in);
	if (!file.good()) {
		error = "Error opening file: '" + logFile->name + "'";
		return(false);
	}
	string s;
	regex rex;
	rex.assign(regexp);
	regex rexip;
	rexip.assign(regexpIP);
	while (!file.fail()) {
		file.getline(buff, 4095);
		if (file.fail())
			break;
		s = buff;
		TLine tl = makeTLine(s, rex, rexip);
		if (tl.line.empty())
			continue;
		matchedLines.push_back(tl);
		if (maxLines && matchedLines.size() >= maxLines)
			break;
	}
	file.close();
	return(true);
}

///////////////////////////////////////////////////////////////////////////////
void	TestFilter::output(http::server::reply& rep) {
	rep.content = s_xml_preamble;
	rep.content += xmlToken(s_matched);
	if (!error.empty()) {
		string s = "<span class=\"mle\">";
		s += error;
		s += "</span>";
		rep.content += toXml(s_line, urlencode(s), 1);
	} else {
		BOOST_FOREACH(TLine tline, matchedLines) {
			output(rep, tline);
		}
	}
	rep.content += xmlTokenE(s_matched);

}

void TestFilter::output(reply& rep, const TLine tline) {
	string s = tline.toXML();
	rep.content += toXml(s_line, urlencode(s), 1);
}

string TLine::toXML() const {
	string s = line;
	if (!length && !ipLength) {
		s = line;
	} else if (length && !ipLength) {
		s = line.substr(0, start);
		s += "<span class=\"mlm\">";
		s += line.substr(start, length);
		s += "</span>";
		s += line.substr(start+length);
	} else if (ipLength && !length) {
		s = line.substr(0, ipStart);
		s += "<span class=\"mli\">";
		s += line.substr(ipStart, ipLength);
		s += "</span>";
		s += line.substr(ipStart+ipLength);
	} else {
		if (ipStart < start) {
			s = "";
			if (ipStart > 0)
				s = line.substr(0, ipStart);
			s += "<span class=\"mli\">";
			s += line.substr(ipStart, ipLength);
			s += "</span>";
			if (start > ipStart + ipLength)
				s += line.substr(ipStart + ipLength, start-(ipStart + ipLength));
			s += "<span class=\"mlm\">";
			s += line.substr(start, length);
			s += "</span>";
			if (start + length < line.length())
				s += line.substr(start + length);
		} else {
			s = "";
			if (start > 0)
				s = line.substr(0, start);
			s += "<span class=\"mlm\">";
			s += line.substr(start, length);
			s += "</span>";
			if (ipStart > start + length)
				s += line.substr(start + length, ipStart-(start + length));
			s += "<span class=\"mli\">";
			s += line.substr(ipStart, ipLength);
			s += "</span>";
			if (ipStart + ipLength < line.length())
				s += line.substr(ipStart + ipLength);

		}
	}
	return(s);
}

} // namespace berzerk
